# -*- coding: utf-8 -*-
"""
@author: Peter Morgan <pete@daffodil.uk.com>
"""

## This simplifies the import...
## and will allows for pyside or alike later

from PyQt5 import QtCore
from PyQt5.QtCore import Qt

from PyQt5 import QtGui
from PyQt5 import QtWidgets
from PyQt5 import QtChart
from PyQt5 import QtNetwork
from PyQt5 import QtWebEngineWidgets
#from PyQt5 import QtSql


from PyQt5.QtCore import pyqtSignal, pyqtSlot

