# -*- coding: utf-8 -*-




version = "0.1-alpha"

##==============================================
## Below  vars are initialsed in MainWindow.__init__

args = None
"""Command args from :ref:`ogt-desktop.py` """

mainWin = None
"""Pointer to the :class:`~ogtgui.main_window.MainWindow`"""

settings = None
"""Global :class:`settings.Settings` instance"""

server = None
"""Pointer to http client"""

ags = None
# """Pointer to an :class:`ogtgui.ags_models.AgsObject` instance"""
